/*
	GWEN
	Copyright (c) 2010 Facepunch Studios
	See license in Gwen.h
*/

#pragma once
#ifndef GWEN_CONFIG_H
#define GWEN_CONFIG_H

//
// Disables animation functions.
//
//#define GWEN_NO_ANIMATION 

#endif
